pmc.default <-
function(data, alpha = 0.05, perd = 24, GrpID = NA){
  
  vars <- tolower(colnames(data))
  idx <- which(vars %in% c("pr", "mesor", "amp", "phi"))
  colnames(data)[idx] <- vars[idx]
  
  if(!all(c("pr", "mesor", "amp", "phi") %in% vars)){
    if(!"pr" %in% vars){
      data$pr <- NA
    }
    if(!"mesor" %in% vars){
      data$mesor <- NA
    }
    if(!"amp" %in% vars){
      data$amp <- 1
    }
    if(!"phi" %in% vars){
      stop("data must contain 'phi' for PMC")   
    }
  }
  
  if(is.null(GrpID) | any(is.na(GrpID))){
    out <- pmc_internal(data, alpha, perd)    
  }else{
    #browser()
    data$pr<-as.numeric(data$pr)
    data$mesor<-as.numeric(data$mesor)
    data$amp<-as.numeric(data$amp)
    data$phi<-as.numeric(data$phi)
    # some quick input checks and define splitting factor
    if(is.character(GrpID)){
      if(!is.factor(data[,GrpID])){
        data[,GrpID] <- factor(data[,GrpID])
      }
      split_factor <- data[,GrpID]
      rownames <- NULL
      # rownames <- unique(data[,GrpID])
    }else{
      print(class(GrpID))
      assert_is_numeric(GrpID)
      if(sum(GrpID) > nrow(data)){
        stop("Group Indices exceed data rows")
      }
      if(sum(GrpID) < nrow(data)){
        warning("Group Indices are less than number of rows. \nAnalysis will omit some rows")
      }
      split_factor <- rep(1:length(GrpID), GrpID)
      if(!is.null(names(GrpID))){
        rownames <- names(GrpID)
      }else{
        rownames <- NULL
      }
    }
    
    if(is.null(rownames)){
      rownames <- names(split(data, split_factor))    
    }
    
    # loop over each group and bind results together
    out <- do.call('rbind',
                   lapply(split(data, split_factor), function(x){
                     pmc_internal(x, alpha, perd)
                   })
    )
    row.names(out) <- rownames
  }
  
  
  return(out)
}
